//+------------------------------------------------------------------+
//|                                                  Environment.mqh |
//|                                 Copyright 2015, Vasiliy Sokolov. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Vasiliy Sokolov."
#property link      "http://www.mql5.com"
#include <Prototypes.mqh>
/*
          ,
      .      
        (GetPositionsLongTotal/GetPositionsShortTotal)
          (AveragePrice())
     .
*/

//+------------------------------------------------------------------+
//|     .                       |
//+------------------------------------------------------------------+
class CEnvironment
{
private:
   uint m_magic;                //   .
   int  m_positions_long;       //    .
   int  m_positions_short;      //    .
   datetime m_bar_time_open;    //       NewBarDetected.
   ulong m_last_id;             //   .
   double m_avrg_price;         //     .
   datetime m_first_pos_open;   //    .
   double m_equity_high;        //      . 
   double m_curr_profit;        //    .
   int GetDynamicPips(void);
public:
   CEnvironment();
   void Refresh();
   void SetMagic(uint mg);      
   uint GetMagic(void);
   bool NewBarDetected(void);
   int  GetPositionsTotal(void);
   int  GetPositionsLongTotal(void);
   int  GetPositionsShortTotal(void);
   bool IsMainPosition(void);
   bool IsLastPositionLost(void);
   ulong LastPositionId();
   double AveragePrice(void);
   datetime FirstPosOpen(void);
   double GetHighEquity();
   double CurrentProfit();
   ENUM_DIRECTION_TYPE CurrentDirection();
   bool DoubleEquals(const double a,const double b);
};
//+------------------------------------------------------------------+
//|                                            |
//+------------------------------------------------------------------+
CEnvironment::CEnvironment()
{
//---
   m_magic = 0;
   m_positions_long = 0;
   m_positions_short = 0;
   m_bar_time_open = 0;
   m_last_id = 0;
   m_avrg_price = 0.0;
   m_first_pos_open = 0;
   m_equity_high = 0.0;
   m_curr_profit = 0.0;
//---
}
//+------------------------------------------------------------------+
//|    .                         |
//+------------------------------------------------------------------+
void CEnvironment::SetMagic(uint mg)
{
   m_magic = mg;
};

//+------------------------------------------------------------------+
//|    .                         |
//+------------------------------------------------------------------+
uint CEnvironment::GetMagic(void)
{
   return m_magic;
};

//+------------------------------------------------------------------+
//|   ,           |
//|      .     |
//| .                                                          | 
//+------------------------------------------------------------------+
bool CEnvironment::IsMainPosition(void)
{
//---
   if(TransactionType() != TRANS_HEDGE_POSITION)return false;
   if(HedgePositionGetString(HEDGE_POSITION_SYMBOL) != Symbol())return false;
   if(HedgePositionGetInteger(HEDGE_POSITION_MAGIC) != m_magic)return false;
   return true;
//---
}
//+------------------------------------------------------------------+
//|                                        |
//+------------------------------------------------------------------+
CEnvironment::Refresh(void)
{
//---
   m_positions_long = 0;
   m_positions_short = 0;
   m_avrg_price = 0.0;
   m_first_pos_open = 0;
   m_curr_profit = 0.0;
   m_last_id = 0;
   double count = 0.0;
   double equity = 0.0;
   if(TransactionsTotal() == 0)
      m_equity_high = 0.0;
   FOREACH_POSITION
   {
      if(!TransactionSelect(i))continue;
      if(TransactionType() != TRANS_HEDGE_POSITION)continue;
      if(HedgePositionGetString(HEDGE_POSITION_SYMBOL) != Symbol())continue;
      if(HedgePositionGetInteger(HEDGE_POSITION_MAGIC) != m_magic)continue;
      equity += HedgePositionGetDouble(HEDGE_POSITION_PROFIT_CURRENCY);
      datetime open = (datetime)(HedgePositionGetInteger(HEDGE_POSITION_ENTRY_TIME_EXECUTED_MSC)/1000);
      if(m_first_pos_open == 0 || m_first_pos_open > open)
         m_first_pos_open = open;
      ulong curr_id = HedgePositionGetInteger(HEDGE_POSITION_ENTRY_ORDER_ID);
      double vol = HedgePositionGetDouble(HEDGE_POSITION_VOLUME);
      m_avrg_price += HedgePositionGetDouble(HEDGE_POSITION_PRICE_OPEN)*vol;
      count += vol;
      if(curr_id > m_last_id)
         m_last_id = curr_id;
      IF_LONG
         m_positions_long++;
      IF_SHORT
         m_positions_short++;
   }
   if(equity > m_equity_high)
      m_equity_high = equity;
   m_curr_profit = equity;
   m_avrg_price = DoubleEquals(count, 0.0) ? 0.0 : NormalizeDouble(m_avrg_price/count, Digits());
//---
}
//+------------------------------------------------------------------+
//|     double.                             |
//| RESULT                                                           |
//|    ,                           |
//|      .                                        |
//+------------------------------------------------------------------+
bool CEnvironment::DoubleEquals(const double a,const double b)
  {
//---
   return(fabs(a-b)<=16*DBL_EPSILON*fmax(fabs(a),fabs(b)));
//---
  }
//+------------------------------------------------------------------+
//|     .                  |
//+------------------------------------------------------------------+
int CEnvironment::GetPositionsTotal(void)
{
   return m_positions_long + m_positions_short;
}
//+------------------------------------------------------------------+
//|     .                  |
//+------------------------------------------------------------------+
int CEnvironment::GetPositionsLongTotal(void)
{
   return m_positions_long;
}
//+------------------------------------------------------------------+
//|     .                 |
//+------------------------------------------------------------------+
int CEnvironment::GetPositionsShortTotal(void)
{
   return m_positions_short;
}
//+------------------------------------------------------------------+
//|      .             |
//|   ,  DIRECTION_UNDEFINED                 |
//+------------------------------------------------------------------+
ENUM_DIRECTION_TYPE CEnvironment::CurrentDirection(void)
{
   if(m_positions_long)return DIRECTION_LONG;
   else if(m_positions_short)return DIRECTION_SHORT;
   return DIRECTION_UNDEFINED;   
}
//+------------------------------------------------------------------+
//|     .         |
//|   ,  .                           |
//+------------------------------------------------------------------+
ulong CEnvironment::LastPositionId(void)
{
   if(GetPositionsTotal() == 0)
      m_last_id = 0;
   return m_last_id;
}
//+------------------------------------------------------------------+
//|  ,     .         |
//|     .                              | 
//+------------------------------------------------------------------+
bool CEnvironment::NewBarDetected(void)
{
//---
   datetime time[];
   if(CopyTime(Symbol(), PERIOD_CURRENT, 0, 1, time) > 0 &&
      m_bar_time_open == time[0])
   {
      return false;
   }
   m_bar_time_open = time[0];
   return true;   
//---
}
//+------------------------------------------------------------------+
//|             |
//+------------------------------------------------------------------+
double CEnvironment::AveragePrice(void)
{
   return m_avrg_price;
}

datetime CEnvironment::FirstPosOpen(void)
{
   return m_first_pos_open;
}
//+------------------------------------------------------------------+
//|   equity,       |
//|   .                                         |
//+------------------------------------------------------------------+
double CEnvironment::GetHighEquity(void)
{
   return m_equity_high;
}
//+------------------------------------------------------------------+
//|            |
//| .                                                        |
//+------------------------------------------------------------------+
double CEnvironment::CurrentProfit()
{
   return m_curr_profit;
}